import perfecto
import os
import robot
import inspect
import urllib2
import PerfectoLibrary

# import urlparse
from perfecto import *
from robot.libraries.BuiltIn import BuiltIn
from appium import webdriver
from .keywordgroup import KeywordGroup
from ._devices import _DeviceKeywords
from urllib import quote_plus


class _RestKeywords(KeywordGroup):
    def __init__(self):
        self.bi = BuiltIn()
        self.securityToken=None
        self.user=None
        self.password=None
        self.host=None

    def _perform_rest_request(self,url):
        '''
        :param url:
        :return:
        '''
        return urllib2.urlopen(url).read()
    
    def _exeRestCmd(self,cmd,subcmd,params):
        '''

        :param cmd:
        :param subcmd:
        :param params:
        :return:
        '''
        actions={}
        actions['command'] = cmd
        actions['subcommand'] = subcmd
        if self._check_driver():
            if('deviceId' not in params):
                params['deviceId'] = self.driver.capabilities['devicename']
            svcStr="executions/"+self.driver.capabilities['executionId']
            return self._exe_restops("command",svcStr,actions,params)

    def set_credentials(self,user,password):
        '''

        :param user:
        :param password:
        :return:
        '''
        self.user=user
        self.password=password

    def set_securityToken(self,securityToken):
        '''
        :param securityToken:
        :return:
        '''
        self.securityToken=securityToken
    def set_host(self,host):
        '''
        :param host:
        :return:
        '''
        self.host=host
    def retrieve_Device_Info(self,deviceid):
        '''

        :param deviceid:
        :return:
        '''
        acts={}
        params={}
        svcStr="handsets/"+deviceid
        # self.bi.log_to_console( "in retrieve_Device_Info")
        self.bi.log_to_console(self._exe_restops("info",svcStr,acts,params))
        
    def start_network_virtualization(self,deviceid,profile):
        '''

        :param deviceid:
        :param profile:
        :return:
        '''
        params={}
        params['deviceId']=deviceid
        params['profile']=profile
        self.bi.log_to_console(self._exeRestCmd("vnetwork","start",params))
        
    def update_network_virtualization(self,deviceid,profile):
        '''

        :param deviceid:
        :param profile:
        :return:
        '''
        params={}
        params['deviceId']=deviceid
        params['profile']=profile
        self.bi.log_to_console(self._exeRestCmd("vnetwork","update",params))
        
        
    def stop_network_virtualization(self,deviceid):
        '''

        :param deviceid:
        :return:
        '''
        params={}
        params['deviceId']=deviceid
        self.bi.log_to_console(self._exeRestCmd("vnetwork","stop",params))
        
    def _exe_restops(self,ops,serviceStr,actions,params):
        '''

        :param ops:
        :param serviceStr:
        :param actions:
        :param params:
        :return:
        '''
        user = self.user
        password = self.password
        host = self.host
        securityToken = self.securityToken

        if self._check_driver():
            user=self.driver.capabilities['user'] if user==None else user
            password=self.driver.capabilities['password'] if password==None else password
            host=self.driver.capabilities['host'] if host==None else host
            securityToken=self.driver.capabilities['securityToken'] if securityToken==None else securityToken

        if (  None == securityToken or "" == securityToken):
            authStr="&user=" + user + "&password=" + password
        else:
            authStr="&securityToken=" + securityToken

        actionStr=""
        for key, value in actions.iteritems():
            actionStr=actionStr+"&"+key+"="+value

        paramStr=""
        for key, value in params.iteritems():
            paramStr=paramStr+"&param." + key +"="+quote_plus(value)

        url = "https://" \
            + host \
            + "/services/" \
            + serviceStr \
            + "?operation=" + ops \
            + authStr \
            + actionStr \
            + paramStr
        self.bi.log_to_console( 'url=' + url)

        return self._perform_rest_request(url)





import perfecto
import os
import robot
import inspect
import PerfectoLibrary
import appium
import urllib2
import time
import sys
from perfecto import *
from robot.libraries.BuiltIn import BuiltIn
# from appium import webdriver
from .keywordgroup import KeywordGroup
from ..listeners import *


class _DeviceKeywords(KeywordGroup):
    def __init__(self):
        self.bi = BuiltIn()
        self.reportPdfUrl = ''


    def _check_driver(self):
        # self.bi.log_to_console("_check_driver")

        try:
            aplib = self.bi.get_library_instance('AppiumLibrary')
            self.driver = aplib._current_application()
            self.reportPdfUrl = self.driver.capabilities['reportPdfUrl']
            return True
        except:
            self.bi.log_to_console(
                "Your script is not using Appium Driver, devices keywords will not be able to performed")
            return False

    def install_application(self, repoName, isSensorInstrument):
        if self._check_driver():
            sensorInstrument = 'nosensor'
            if isSensorInstrument.lower() == 'true':
                sensorInstrument = 'sensor'
            params = {}
            params['sensorInstrument'] = sensorInstrument
            params['file'] = repoName
            self.driver.execute_script('mobile:application:install', params)

    def uninstall_application(self, name):
        if self._check_driver():
            params = {}
            params['identifier'] = name
            self.driver.execute_script('mobile:application:uninstall', params)

    def start_application_by_name(self, name):
        if self._check_driver():
            params = {}
            params['identifier'] = name
            self.driver.execute_script('mobile:application:open', params)

    def close_application_by_name(self, name):
        if self._check_driver():
            params = {}
            params['identifier'] = name
            self.driver.execute_script('mobile:application:close', params)

    def open_system_browser(self):
        if self._check_driver():
            params = {}
            params['automation'] = 'os'
            self.driver.execute_script('mobile:browser:open', params)

    def browser_execute_script(self, scriptString):
        if self._check_driver():
            params = {}
            params['script'] = scriptString
            params['timeout'] = '35'
            self.driver.execute_script('mobile:browser:execute', params)

    def browser_execute_repo_script(self, scriptRepoLoc):
        if self._check_driver():
            params = {}
            params['repositoryFile'] = scriptRepoLoc
            params['timeout'] = '35'
            self.driver.execute_script('mobile:browser:execute', params)

    def maximize_window(self):
        if self._check_driver():
            self.driver.maximize_window()

    def scroll_to_element(self, elementxpath):
        if self._check_driver():
            params = {}
            params['element'] = (self.driver.findElement(By.xpath(elementxpath))).GetId()
            params['toVisible'] = 'any'
            self.driver.execute_script('mobile:scroll', params)

    # devices actions
    def rotate(self, state='landscape', method='device'):
        if self._check_driver():
            params = {}
            params['state'] = state
            params['method'] = method
            self.driver.execute_script('mobile:device:rotate', params)

    def browser_execute_script(self, text, timeout='20'):
        if self._check_driver():
            params = {}
            params['script'] = text
            params['timeout'] = timeout
            self.driver.execute_script('mobile:browser:execute', params)

    def drag(self, x1, y1, x2, y2, duration='5'):
        '''
        The touch event coordinates.
        Format - either "x1,y1,x2,y2" or "x1%,y1%,x2%,y2%"
        Coordinate value can be in pixels or in percentage of screen size (0-100).
        For percentage use the % sign. Example - "20%, 25%"
        It is recommended to use the percentage value as it translates to the screen resolution.
        :param duration: The duration, in seconds, for performing the drag operation.
        :return: None
        '''
        if self._check_driver():
            params = {}
            params['location'] = x1 + ',' + y1 + ',' + x2 + ',' + y2
            params['duration'] = duration
            self.driver.execute_script('mobile:touch:drag', params)

    def gesture(self, startx, starty, endx, endy, operation='Zoom', duration='5'):
        '''

        :param startx, starty:The start, touch down, event coordinates.
                Format - "x,y" or "x%,y%"
                Coordinate value can be in pixels or in percentage of screen size (0-100).
                For percentage use the % sign.
                Example - 50%, 50%
                It is recommended to use the percentage value as it does not rely on the screen resolution.
        :param endx,endy:The end, touch up, event coordinates.
                Format - "x,y" or "x%,y%"
                Coordinate value can be in pixels or in percentage of screen size (0-100).
                For percentage use the % sign.
                Example - 50%, 50%
                It is recommended to use the percentage value as it does not rely on the screen resolution.
        :param operation:    The gesture operation type.
                Zoom - performs pinch open operation from the start coordinates to the end coordinates.
                Pinch - performs pinch close operation from the start coordinates to the end coordinates
        :param duration: The duration, in seconds, for performing the operation.
        :return:None
        '''
        if self._check_driver():
            params = {}
            params['start'] = startx + ',' + starty
            params['end'] = endx + ',' + endy
            params['operation'] = operation
            params['duration'] = duration
            self.driver.execute_script('mobile:touch:gesture', params)

    def swipe(self, startx, starty, endx, endy, duration='5'):
        '''
        :param startx, starty:The start, touch down, event coordinates.
                Format - "x,y" or "x%,y%"
                Coordinate value can be in pixels or in percentage of screen size (0-100).
                For percentage use the % sign.
                Example - 50%, 50%
                It is recommended to use the percentage value as it does not rely on the screen resolution.
        :param endx,endy:The end, touch up, event coordinates.
                Format - "x,y" or "x%,y%"
                Coordinate value can be in pixels or in percentage of screen size (0-100).
                For percentage use the % sign.
                Example - 50%, 50%
                It is recommended to use the percentage value as it does not rely on the screen resolution.
        :param duration: The duration, in seconds, for performing the operation.
        :return:None
        '''
        if self._check_driver():
            params = {}
            params['start'] = startx + ',' + starty
            params['end'] = endx + ',' + endy
            params['duration'] = duration
            self.driver.execute_script('mobile:touch:swipe', params)

    def perfecto_tap(self, locx, locy, duration='5'):
        '''
        Note: As a best practice, this function should only be used in extreme circumstances because it is not
        accurate or adaptable to application modifications. Alternatively, screen analysis functions are recommended for robust automated testing.
        :param locx, locy:The touch event coordinates.
                Format - either "x1,y1,x2,y2" or "x1%,y1%,x2%,y2%"
                Coordinate value can be in pixels or in percentage of screen size (0-100).
                For percentage use the % sign. Example - "20%, 25%"
                It is recommended to use the percentage value as it translates to the screen resolution.
        :param duration:The duration, in seconds, for performing the touch operation.
                Use this to perform a "long-press".
        :return: none
        '''
        if self._check_driver():
            params = {}
            params['location'] = locx + ',' + locy
            params['duration'] = duration
            self.driver.execute_script('mobile:touch:tap', params)

    def trackball_roll(self, distance='0,0'):
        '''
        :param distance: The distance from the last location coordinates.
                Format - x,y
                Note: This command receives an x,y sequence relative to the last location coordinates that are not absolute coordinates on the screen.
        :return: none
        '''
        if self._check_driver():
            params = {}
            params['distance'] = distance
            self.driver.execute_script('mobile:trackball:roll', params)

    # Visual Analysis Functions
    def button_image_click(self, label, threhold=80):
        '''
        :param label: Repository path to image file (png, jpg). The image that appears on, or related to, the button
        :param threhold:The acceptable match level percentage, between 20 and 100.
                Too low values can lead to a false positive result, while too high values can lead to a false negative result.
        :return: none
        '''
        if self._check_driver():
            params = {}
            params['label'] = label
            params['threshold'] = threhold
            params['imageBounds.needleBound'] = 30
            self.driver.execute_script('mobile:button-image:click', params)

    def button_text_click(self, label, ignorecase='true'):
        '''
        :param label: Repository path to image file (png, jpg). The image that appears on, or related to, the button
        :param threhold:The acceptable match level percentage, between 20 and 100.
                Too low values can lead to a false positive result, while too high values can lead to a false negative result.
        :return: none
        '''
        if self._check_driver():
            if ignorecase.lower() == 'false':
                ignorecase = 'case'
            else:
                ignorecase = 'nocase'
            params = {}
            params['label'] = label
            params['ignorecase'] = ignorecase
            params['threshold'] = 80
            self.driver.execute_script('mobile:button-text:click', params)

    def find_image_in_screen(self, content, context='body'):
        '''
        :param content:The image to search for.
                    The image can be taken from the device screen using the preview tool or selected from the media repository.
                    If selected from the repository the image must be a JPEG, PNG or BMP file.
        :param context: all | body    Defines the screen region where to look for the needle.
        :return: String indicating success ("true") or failure ("false")
        '''
        if self._check_driver():
            params = {}
            params['content'] = content
            params['context'] = context
            return self.driver.execute_script('mobile:image:find', params)
        return False

    def find_text_in_screen(self, content, context='body'):
        '''
        :param content:The text to search for.
                It is recommended to use the entire string searched for, to ensure that if the OCR misses a few characters, the needle will still be found.
                The text-match algorithm is less error-prone when the provided string is longer.
        :param context: all | body    Defines the screen region where to look for the needle.
        :return: String indicating success ("true") or failure ("false")
        '''
        if self._check_driver():
            params = {}
            params['content'] = content
            params['context'] = context
            return self.driver.execute_script('mobile:text:find', params)
        return False

    def device_info(self, property):
        '''
        :param property: possible values:
         manufacturer | model | phoneNumber | deviceId | resolution |
        resolutionWidth | resolutionHeight | os | osVersion | firmware | location | network | distributer | language | imsi | nativeImei | wifiMacAddress |
        cradleId | status | inUse | description | position | method | rotation | locked | roles |
        currentActivity |
        currentPackage | all | hasAudio | automationInfrastructure
        :return: String indicating success ("true") or failure ("false")
         '''
        if self._check_driver():
            params = {}
            params['property'] = property
            return self.driver.execute_script('mobile:device:info', params)
        return False


    def keep_session_alive(self, time_in_seconds):
        '''
        THis keyword will keep the remote session to be alive for the expected length of time

        :param time_in_seconds:
                to keep the remote session alive for how many seconds
        :return: String indicating success ("true") or failure ("false")
        '''
        if self._check_driver():
            for i in range(0, int(time_in_seconds) + 1, 60):
                time.sleep(60)
                self.device_info('model')
            return True
        return False



# -*- coding: utf-8 -*-


from ._devices import _DeviceKeywords
from ._rest_acts import _RestKeywords
from ._general  import _GeneralKeywords

__all__ = ["_DeviceKeywords","_RestKeywords","_GeneralKeywords"]



#!/usr/bin/env python

from os.path import abspath, dirname, join
from setuptools import setup, find_packages

ROOT = dirname(abspath(__file__))

version_file = join(ROOT, 'PerfectoLibrary', 'version.py')
exec (compile(open(version_file).read(), version_file, 'exec'))

setup(name='perfectolibrary',
      version=VERSION,
      description='Robot Framework Mobile app testing library for using Perfecto Cloud',
#       long_description=open(join(ROOT, 'README.rst')).read(),
      author='Jack Deng',
      author_email='JackD@perfectoMobile.com',
      url='https://github.com/PerfectoMobileSA/Robotframework-PerfectoLibrary',
      license='MIT License',
      keywords='robotframework testing Perfecto mobile appium webdriver app android ios',
      platforms='any',
      classifiers=[
          "Development Status :: 3 - Alpha",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Topic :: Software Development :: Testing",
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6'
      ],
      install_requires=[
          'decorator >= 3.3.2',
          'robotframework >= 2.6.0',
          'docutils >= 0.8.1',
          'Appium-Python-Client >= 0.20',
          'selenium >= 2.47.1',
          'robotframework-appiumlibrary >= 1.3.7',
          'robotframework-seleniumlibrary >= 3.0.0',
          'robotframework-selenium2library >= 1.6.0',
          'Selenium2LibraryExtension >= 1.1.0',
          'perfecto >= 3.0.3',
          'six >= 1.11.0'
      ],
      packages=find_packages(exclude=["demo", "docs", "tests", ]),
      include_package_data=True,
      )



import perfecto
import os
import robot
import inspect
import PerfectoLibrary
import appium
import urllib2
import traceback
import time
import sys
import subprocess
from perfecto import *
from robot.libraries.BuiltIn import BuiltIn
# from appium import webdriver
from .keywordgroup import KeywordGroup
from ..listeners import *


class _GeneralKeywords(KeywordGroup):
    def __init__(self):
        self.bi = BuiltIn()
        self.reportPdfUrl = ''

    def init_driver(self):
        self._check_driver()

    def _check_driver(self):

        try:
            aplib = self.bi.get_library_instance('AppiumLibrary')
            self.driver = aplib._current_application()
            self.active = True
        except:
            try:
                aplib = self.bi.get_library_instance('SeleniumLibrary')
                self.driver = aplib.driver
                # self.bi.log_to_console(aplib)
                self.active = True
            except:
                try:
                    aplib = self.bi.get_library_instance('Selenium2Library')
                    self.driver = self.driver = aplib._current_browser()
                    self.active = True
                except:
                    try:
                        aplib = self.bi.get_library_instance('Selenium2LibraryExtension')
                        self.driver = self.driver = aplib._current_browser()
                        self.active = True
                    except:
                        self.active = False
        if self.driver != None:
            self.reportPdfUrl=self.driver.capabilities['reportPdfUrl']

    def enable_proxy(self, str):
        proxy = str
        os.environ['http_proxy'] = proxy
        os.environ['HTTP_PROXY'] = proxy
        os.environ['https_proxy'] = proxy
        os.environ['HTTPS_PROXY'] = proxy

    def disable_proxy(self):
        proxy = ""
        os.environ['http_proxy'] = proxy
        os.environ['HTTP_PROXY'] = proxy
        os.environ['https_proxy'] = proxy
        os.environ['HTTPS_PROXY'] = proxy
        try:
            del os.environ['http_proxy']
            del os.environ['https_proxy']
            del os.environ['HTTP_PROXY']
            del os.environ['HTTPS_PROXY']
        except:
            pass


    def perfectoconnect_start(path_to_perfectoconnectexe,cloud,sec_token,proxyuser=None,proxypass=None,proxyserverip=None,proxyport=None):
        """

         :param path_to_perfectoconnectexe: The relative or absolute path to the perfectoconnect executable
        :param cloud: the cloud name, for instance somecloud.perfectomobile.com
        :param sec_token: the security token to aithenticate you to access the cloud and device
        :param proxyuser: the proxy username
        :param proxypass: the proxy password
        :param proxyserverip: the ipaddress of the proxy server
        :param proxyport: the port number of the proxy server
        :return: on seccss, it returns the tunnelId. You can then add tunnelId as the capability when launching perfectodriver
        """

        cmd = ''
        bi=BuiltIn()
        if proxyuser is not None and proxypass is not None and proxyserverip is not None and proxyport is not None:
            # bi.log_to_console("in the first if")
            cmd = [path_to_perfectoconnectexe, 'start', '--cloudurl='+cloud, '--securitytoken='+sec_token, '--outgoingproxyuser='+proxyuser,
                   '--outgoingproxypass='+proxypass, '--outgoingproxyip='+proxyserverip, '--outgoingproxyport='+proxyport]

        elif proxyuser is None and proxypass is None and proxyserverip is not None and proxyport is not None:
            cmd = [path_to_perfectoconnectexe,
                   'start', '--cloudurl='+cloud, '--securitytoken='+sec_token, '--outgoingproxyip=' + proxyserverip, '--outgoingproxyport=' + proxyport]
        else:
            cmd = [path_to_perfectoconnectexe, 'start', '--cloudurl='+cloud, '--securitytoken='+sec_token]
        # bi.log_to_console("path_to_perfectoconnectexe=" + path_to_perfectoconnectexe)
        # bi.log_to_console(', '.join(cmd))
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        o, e = proc.communicate()
        # bi.log_to_console("output="+o.decode('ascii'))
        # bi.log_to_console("err="+e.decode('ascii'))
        return o.decode('ascii').strip()

    def perfectoconnect_stop(path_to_perfectoconnectexe):
        '''
            :param path_to_perfectoconnectexe: path_to_perfectoconnectexe: The relative or absolute path to the perfectoconnect executable
            :return: None
        '''
        cmd = [path_to_perfectoconnectexe, 'stop']
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        o, e = proc.communicate()

    def maximize_window(self):
        if self._check_driver():
            self.driver.maximize_window()

    def download_summary_pdf_report(self, reportpath, sectoken, executionID=None, jobName=None, jobNumber=None,
                                    tags=None):
        '''

        :param reportpath: the local path where you want to store the pdf reports
                sectoken: perfecto securitytoken to access the report
                executionID: the executionID of the reports
                jobName: the jobName of the reports
                jobID: the jobID of the reports
                tags: the tags of the reports
        :return: return false if anything go wrong
        '''
        exeRptUrl = self.reportPdfUrl
        rptQuery = ''
        if self.reportPdfUrl != '':

            if executionID == None and jobName == None and jobNumber == None and tags == None:
                exeRptUrl = self.reportPdfUrl
            else:
                if executionID != None:
                    if rptQuery == '':
                        rptQuery = 'externalId[0]=' + executionID
                    else:
                        rptQuery = rptQuery + '&' + 'externalId[0]=' + executionID
                if jobName != None:
                    if rptQuery == '':
                        rptQuery = 'jobName[0]=' + jobName
                    else:
                        rptQuery = rptQuery + '&' + 'jobName[0]=' + jobName
                if tags != None:
                    if rptQuery == '':
                        rptQuery = 'tags[0]=' + tags
                    else:
                        rptQuery = rptQuery + '&' + 'tags[0]=' + tags
                if jobNumber != None:
                    if rptQuery == '':
                        rptQuery = 'jobNumber[0]=' + jobNumber
                    else:
                        rptQuery = rptQuery + '&' + 'jobNumber[0]=' + jobNumber
                exeRptUrl = self.reportPdfUrl.split('pdf?')[0] + 'pdf?' + rptQuery
                # self.bi.log_to_console(exeRptUrl)

            time.sleep(10)  # have to sleep for 10 seconds
            try:
                headers = {'PERFECTO-AUTHORIZATION': sectoken, }
                req = urllib2.Request(exeRptUrl, None, headers)
                rp = urllib2.urlopen(req)
                with open(reportpath + self.reportPdfUrl.rsplit('=', 1)[-1] + '.pdf', 'wb') as output:
                    output.write(rp.read())
                    output.close()
                return True
            except:
                self.bi.log_to_console(traceback.print_exc())
                return False
        self.bi.log_to_console("empty with " + self.reportPdfUrl)
        return False



import os
import robot
import inspect
import pdb
import sys
import PerfectoLibrary
from perfecto import *
import traceback
# from SeleniumLibrary import SeleniumLibrary
# from Selenium2Library import Selenium2Library
# from AppiumLibrary import AppiumLibrary
# from Selenium2LibraryExtension import Selenium2LibraryExtension
from robot.libraries.BuiltIn import BuiltIn


class _PerfectoListener(object):
    ROBOT_LISTENER_API_VERSION = 2
    ROBOT_LIBRARY_SCOPE = "GLOBAL"
    driver = ''
    projectname = 'Robotframework Test Project'
    projectversion = '1.0'
    jobname = 'Robotframework Test Job'
    jobnumber = 1

    def __init__(self):
        # pdb.Pdb(stdout=sys.__stdout__).set_trace()
        self.ROBOT_LIBRARY_LISTENER = self
        self.bi = BuiltIn()
        self.reporting_client = None
        self.active = False
        self.stop_reporting = False
        self.tags = ''
        self.longname = 'Robotframework Script'
        self.id = 's1-t1'
        self.running = False
        self.suitesetup = False
        self.setupclient = None

    def init_listener(self, projectname=None, projectversion=None, jobname=None, jobnumber=None):
        """
        This key word helps to initialize the listener with proper project info
        :param projectname: current project name
        :param projectversion: current project version
        :param jobname: the CI job name
        :param jobnumber: the CI job number
        :return:
        """
        if projectname != None:
            self.projectname = projectname
        if projectversion != None:
            self.projectversion = projectversion
        if jobname != None:
            self.jobname = jobname
        if jobnumber != None:
            self.jobnumber = long(float(jobnumber))

    def _start_suite(self, name, attrs):
        #         pdb.Pdb(stdout=sys.__stdout__).set_trace()

        if not self.active:
            self._get_execontext()

    def _start_test(self, name, attrs):
        # pdb.Pdb(stdout=sys.__stdout__).set_trace()
        self.id = attrs['id']
        self.longname = self.bi.get_variable_value('${TEST NAME}')
        self.tags = attrs['tags']
        #         if not self.active:
        self._get_execontext()
        if self.active and self.reporting_client != None and self.running == False:
            self._suitesetup_result()
            self.reporting_client.test_start(self.longname, TestContext([], self.tags))
            self.running = True

    def _suitesetup_result(self):
        if self.suitesetup:
            if self.bi.get_variable_value('${TEST STATUS}') == 'FAIL':
                self.setupclient.test_stop(
                    TestResultFactory.create_failure(self.bi.get_variable_value('${TEST MESSAGE}')))
            else:
                self.setupclient.test_stop(TestResultFactory.create_success())
            self.suitesetup = False

    def _start_keyword(self, name, attrs):
        try:
            if not self.active:
                self._get_execontext()

            if self.active and self.reporting_client != None and self.stop_reporting != True \
                    and self.running == False and "tear" not in attrs['type'].lower():
                if self.bi.get_variable_value('${TEST NAME}') != None:
                    self._suitesetup_result()
                    self.reporting_client.test_start(self.bi.get_variable_value('${TEST NAME}'),
                                                     TestContext([], self.tags))

                else:
                    self.reporting_client.test_start('Suite Setup', TestContext([], self.tags))
                    self.setupclient = self.reporting_client
                    self.suitesetup = True
                self.running = True

            # pass
            if self.active and self.reporting_client != None and self.stop_reporting != True \
                    and "tear" in attrs['type'].lower():
                if self.bi.get_variable_value('${TEST STATUS}') == 'FAIL':
                    self.reporting_client.test_stop(
                        TestResultFactory.create_failure(self.bi.get_variable_value('${TEST MESSAGE}')))
                else:
                    self.reporting_client.test_stop(TestResultFactory.create_success())
                self.stop_reporting = True
                self.running = False

            if self.active and self.reporting_client != None and self.stop_reporting != True \
                    and "comment" not in attrs['kwname'].lower() \
                    and "excel" not in attrs['kwname'].lower() \
                    and "csv" not in attrs['kwname'].lower() \
                    and "sheet" not in attrs['kwname'].lower() \
                    and "cell" not in attrs['kwname'].lower() \
                    and "column" not in attrs['kwname'].lower() \
                    and ("keyword" in attrs['type'].lower() \
                         or "setup" in attrs['type'].lower()) \
                    and "builtin" not in attrs['libname'].lower() \
                    and "collections" not in attrs['libname'].lower() \
                    and "dialogs" not in attrs['libname'].lower() \
                    and "easter" not in attrs['libname'].lower() \
                    and "operatingsystem" not in attrs['libname'].lower() \
                    and "process" not in attrs['libname'].lower() \
                    and "screenshot" not in attrs['libname'].lower() \
                    and "string" not in attrs['libname'].lower() \
                    and "telnet" not in attrs['libname'].lower() \
                    and "xml" not in attrs['libname'].lower() \
                    and "excellibrary" not in attrs['libname'].lower() \
                    and "selenium" not in attrs['libname'].lower() \
                    and "database" not in attrs['libname'].lower() \
                    and "bytes" not in attrs['kwname'].lower() \
                    and "appium" not in attrs['libname'].lower():
                self.reporting_client.step_start(attrs['kwname'] + ' ' + ' '.join(attrs['args']))

        except Exception as e:
            self.bi.log_to_console(traceback.format_exc())
            pass

    #     def _end_keyword(self, name, attrs):
    #         if "setup" in attrs['type'].lower() \
    #             and ("selenium" in attrs['libname'].lower() \
    #             or "appium"  in attrs['libname'].lower()):
    #                 self._get_execontext()

    def _get_execontext(self):
        # self.bi.log_to_console("_get_execontext")
        try:
            aplib = self.bi.get_library_instance('AppiumLibrary')
            self.driver = aplib._current_application()
            # self.bi.log_to_console(aplib)
            self.active = True
        except:
            try:
                aplib = self.bi.get_library_instance('SeleniumLibrary')
                self.driver = aplib.driver
                # self.bi.log_to_console(aplib)
                self.active = True
            except:
                try:
                    aplib = self.bi.get_library_instance('Selenium2Library')
                    self.driver = self.driver = aplib._current_browser()
                    self.active = True
                except:
                    try:
                        aplib = self.bi.get_library_instance('Selenium2LibraryExtension')
                        self.driver = self.driver = aplib._current_browser()
                        self.active = True
                    except:
                        self.active = False

        if self.active:
            self.execontext = PerfectoExecutionContext(self.driver, self.tags, Job(self.jobname, self.jobnumber),
                                                       Project(self.projectname, self.projectversion))
            self.reporting_client = PerfectoReportiumClient(self.execontext)

    def _end_test(self, name, attrs):
        if self.stop_reporting != True:
            try:
                if attrs['status'] == "PASS":
                    self.reporting_client.test_stop(TestResultFactory.create_success())
                else:
                    self.reporting_client.test_stop(TestResultFactory.create_failure(attrs['message']))
            except Exception as e:
                # trace = traceback.format_exc()
                # self.bi.log_to_console(trace)
                pass
        self.stop_reporting = False
        self.reporting_client = None
        self.active = False
        self.running = False


# -*- coding: utf-8 -*-

import os
from PerfectoLibrary.keywords import *
from PerfectoLibrary.listeners import *
from PerfectoLibrary.version import VERSION

__version__ = VERSION


class PerfectoLibrary(
    _DeviceKeywords,
    _GeneralKeywords,
    _RestKeywords,
    _PerfectoListener,
):
    """

    """

    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
    ROBOT_LIBRARY_VERSION = VERSION

    def __init__(self):
        """
        """
        for base in PerfectoLibrary.__bases__:
            base.__init__(self)

    def __init__(self,projectname=None,projectversion=None,jobname=None,jobnumber=None):
        """
        """
        for base in PerfectoLibrary.__bases__:
            base.__init__(self)
        _PerfectoListener.init_listener(self,projectname,projectversion,jobname,jobnumber)
